﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Xml;

namespace GeoSerializerTester
{
    public partial class Form1 : Form
    {
        Proj4Projection proj4Projection = new Proj4Projection();

        public Form1()
        {
            InitializeComponent();



            winformsMap1.MapUnit = GeographyUnit.Meter;
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);
            winformsMap1.CurrentExtent = new RectangleShape(-180, 90, 180, -90);

            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            proj4Projection.Open();

            graticuleAdornmentLayer.Open();
            graticuleAdornmentLayer.Projection = proj4Projection;
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.White);
            graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);

            winformsMap1.Refresh();
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            // Select the file.  If it's okay, then open it.
            openFileDialog.Filter = "Text Files (*.txt)|*.txt";
            openFileDialog.FileName = "";
            openFileDialog.Title = "Open a Text File";

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                textEMUFile.Text = openFileDialog.FileName;
            }
            else
            {
                return;
            }

            // Clear the overlays
            winformsMap1.Overlays.Clear();
            winformsMap1.Refresh();

            int utmZone = 15;
            string utmHemi = "north";
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4Projection.ExternalProjectionParametersString = string.Format("+proj=utm +zone={0} +{1} +ellps=WGS84 +datum=WGS84 +units=m +no_defs", utmZone, utmHemi);//UTM

            // Process the file
            FileStream input = new FileStream(textEMUFile.Text, FileMode.Open, FileAccess.Read);

            // Deserialize
            Collection<object> allData = new Collection<object>();
            using (input)
            {
                allData = ReadSerializedAll(input);
            }

            if (allData.Count == 0)
            {
                return;
            }

            // Add overlays to map and refresh
            Collection<Overlay> fullOverlays = (Collection<Overlay>)allData[0];

            foreach (var overlay in fullOverlays)
            {
                LayerOverlay layerOverlay = (LayerOverlay)overlay;
                foreach (Layer layer in layerOverlay.Layers)
                {
                    Debug.Print("Layer Type: " + layer.GetType().ToString() + " Layer Name: " + layer.Name);

                    // Now try and refresh the layer
                    try
                    {
                        layer.Close();
                        layer.Open();
                    }
                    catch (FileNotFoundException)
                    {
                        // If we have an exception about the filename not existing, then we don't load this layer because
                        // they didn't bring the shape with them.
                        MessageBox.Show(layer.Name);
                    }

                }

                try
                {
                    winformsMap1.Overlays.Add(overlay.Name, overlay);

                    SetCurrentExtent(winformsMap1, (LayerOverlay)overlay);

                }
                catch (Exception exp)
                {
                    MessageBox.Show(exp.Message, "Overlay Not Added", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }


            winformsMap1.Refresh();

            MessageBox.Show("Operation Complete");

        }

        public static void SetCurrentExtent(WinformsMap winformsMap, LayerOverlay layerOverlay)
        {
            RectangleShape currentBoundingBox = GetFullExtent(winformsMap.Width, winformsMap.Height, layerOverlay);
            if (currentBoundingBox != null)
            {
                winformsMap.CurrentExtent = currentBoundingBox;
                winformsMap.Refresh();
            }
        }

        public static RectangleShape GetFullExtent(float screenWidth, float screenHeight, LayerOverlay layerOverlay)
        {
            RectangleShape boundingBox = null;

            int counter = 0;
            foreach (Layer layer in layerOverlay.Layers)
            {

                if (layer.HasBoundingBox && counter > 0)
                {
                    if (!layer.IsOpen) { layer.Open(); }
                    if (boundingBox == null)
                    {
                        boundingBox = layer.GetBoundingBox();
                    }
                    else
                    {
                        boundingBox.ExpandToInclude(layer.GetBoundingBox());
                    }
                }
                counter++;
            }

            if (boundingBox != null)
            {
                boundingBox = MapEngine.GetDrawingExtent(boundingBox, screenWidth, screenHeight);
            }

            return boundingBox;
        }

        public static Collection<object> ReadSerializedAll(FileStream input)
        {
            GeoSerializer reader = new GeoSerializer();
            Collection<object> allData = new Collection<object>();

            try
            {
                GlobalSerializer overlayRecord = (GlobalSerializer)reader.Deserialize(input);
                allData.Add(overlayRecord.FullOverlays);
            }
            catch (SerializationException serEx)
            {
                MessageBox.Show(serEx.Message, "SerializationException Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (InvalidCastException castEx)
            {
                MessageBox.Show(castEx.Message, "InvalidCastException Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (ArgumentException argEx)
            {
                MessageBox.Show(argEx.Message, "ArgumentException Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (XmlException xmlError)
            {
                MessageBox.Show(xmlError.Message, "XML Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return allData;
        }
    }
}
